"""
Overview:
    System requirements conditions for unittest.
    Can be used on ``unittest.skipUnless``, ``pytest.mark.skipUnless``, etc.
"""
from .cmd import *
from .expr import *
from .git import *
